"""
本作品采用知识共享署名-非商业性使用-相同方式共享 4.0 国际许可协议进行许可。
当您使用本作品相关图纸或源码等资料时，视为您同意并遵循此协议内容。

项目开源地址：https://www.laserblock.cn/thread-23573-1-1.html

作者：李嘉泳
邮箱：17620745509@126.COM
BiliBili：https://space.bilibili.com/106497171

Laserblock开源社区：https://www.laserblock.cn/
"""
print("IMPORTING CORE")
# mPythonType:0
from mpython import *
from servo import Servo
import printer_gui_menu as menu
import printer_dot_servo as dot
import printer_gui as gui
import printer_stepper as stepper
import printer_file_system as fil
import os
import math
import utime  # ---GUI---
import printer_screen_saver as saver
import printer_input as inpt
import config as cfg
import framebuf


class file_struct:
    def __init__(self):
        self.size = 0
        self.width = 0
        self.height = 16
        self.byte_buffer = 0
        self.frame_buffer = 0


class print_data_struct:
    def __init__(self):
        self.printing_x_number = 0
        self.printing_y_number = 0
        self.start_time = 0
        self.finish_time = 0
        self.print_time = 0


class arm_servo_struct:
    def __init__(self):
        self.min_angel = cfg.cfg["core"]["arm_servo"]["min_angel"]
        self.max_angel = cfg.cfg["core"]["arm_servo"]["max_angel"]
        self.steps_delay = cfg.cfg["core"]["arm_servo"]["delay"]
        self.dead_zone_angel = cfg.cfg["core"]["arm_servo"]["dead_zone"]
        self.dead_zone_delay = 0
        self.current_step = 0
        self.angel = [cfg.cfg["core"]["arm_servo"]["min_angel"] for i in range(16)]


file = file_struct()
print_data = print_data_struct()
arm_servo = arm_servo_struct()

servo = Servo(1, min_us=500, max_us=2500, actuation_range=180)


def arm_move_to_step(step, delay=True, dead_zone_fix=False):
    global arm_servo
    if step < 0 or step > 15:
        print("错误：指定步数超出范围：", step)
        return
    if arm_servo.current_step == step:
        return
    if not dead_zone_fix:
        servo.write_angle(arm_servo.angel[step])
    else:
        servo.write_angle(arm_servo.angel[step] - arm_servo.dead_zone_angel)
    servo_move_steps = arm_servo.current_step - step
    if servo_move_steps < 0:
        servo_move_steps *= -1
    if delay:
        time.sleep_ms(arm_servo.steps_delay * servo_move_steps)
    arm_servo.current_step = step


arm_servo.current_step = -1
arm_move_to_step(0)


def print_hex_file(file_path):
    global file, print_data, arm_servo
    if not gui.print_content_confirm(file_path):
        return
    print("开始打印文件:", file_path)
    file.size = os.stat(file_path)[6]
    print("文件大小：" + str(file.size) + "byte")
    print("正在缓存文件")
    file.byte_buffer = fil.get_byte_from_file(file_path)
    file.width = int(((file.size * 8) / file.height))
    print(
        "图像高：" + str(file.height),
        "bit , " + str(file.height / 8) + "byte",
    )
    print(
        "图像宽：" + str(file.width),
        "bit , " + str(int(file.width / 8)) + "byte",
    )
    file.frame_buffer = framebuf.FrameBuffer(
        bytearray(file.byte_buffer), file.width, file.height, framebuf.MONO_HLSB
    )
    print("正在初始化硬件数据")
    steps_per_angel = (arm_servo.max_angel - arm_servo.min_angel) / (file.height - 1)
    dot.steps = dot.ANGEL_FIX / (file.height - 1)
    for check_precision_number in range(file.height):
        arm_servo.angel[check_precision_number] = (
            round(arm_servo.min_angel + check_precision_number * steps_per_angel)
        ) + cfg.cfg["core"]["arm_servo"]["angel_fix"][check_precision_number]
    print("摆臂舵机最大角度：" + str(arm_servo.max_angel))
    print("摆臂舵机最小角度：" + str(arm_servo.min_angel))
    print("摆臂舵机步距角度：" + str(steps_per_angel))
    print("打点舵机步距角度：" + str(dot.steps))
    print("摆臂舵机步数角度转换数组：", arm_servo.angel)
    oled.fill(0)  # ---GUI---
    gui.auto_show_text(1, "正  在  打  印")  # ---GUI---
    gui.screen_fresh_percent(0)  # ---GUI---
    print_data.start_time = utime.ticks_ms()  # ---GUI---
    oled.show()  # ---GUI---
    """
    打印核心
    """
    for print_data.printing_x_number in range(file.width):
        dot.set_dot(False)
        for print_data.printing_y_number in range(file.height):
            # handle_pause()
            if (
                file.frame_buffer.pixel(
                    print_data.printing_x_number, print_data.printing_y_number
                )
                == 0
            ):
                continue
            arm_move_to_step(print_data.printing_y_number)
            dot.dot_once()
        dot.dot_standby()
        if not print_data.printing_x_number == file.width - 1:
            for prepare_y_number in range(file.height):
                if (
                    file.frame_buffer.pixel(
                        print_data.printing_x_number + 1, prepare_y_number
                    )
                    == 1
                    and not arm_servo.current_step == prepare_y_number
                ):
                    if prepare_y_number > arm_servo.current_step:
                        arm_move_to_step(
                            prepare_y_number, delay=False, dead_zone_fix=False
                        )
                    else:
                        arm_move_to_step(
                            prepare_y_number, delay=False, dead_zone_fix=True
                        )
                    break
        stepper.steps()
        servo.write_angle(arm_servo.angel[arm_servo.current_step])
        gui.screen_fresh_percent(
            print_data.printing_x_number / file.width * 100
        )  # ---GUI---
    """
    打印核心结束
    """
    print_data.finish_time = utime.ticks_ms()  # ---GUI---
    print("打印完成")
    oled.fill(0)  # ---GUI---
    gui.auto_show_text(1, "打  印  完  成")  # ---GUI---
    print_data.print_time = round(
        (print_data.finish_time - print_data.start_time) / 1000
    )
    print_time_text = (
        "耗时："
        + str(gui.convert_time(print_data.print_time)[1])
        + "分"
        + str(gui.convert_time(print_data.print_time)[2])
        + "秒"
    )
    print(print_time_text)
    gui.auto_show_text(2, print_time_text)  # ---GUI---
    dot.dot_standby()
    arm_move_to_step(0)
    stepper.steps(stepper.stepper.step_pwm_count * file.height)
    saver.reset()  # ---GUI---


def handle_pause():  # ---GUI---
    if inpt.is_pressed(inpt.b):
        oled.fill(0)
        gui.auto_show_text(1, "任务已暂停")
        # gui.auto_show_text(2, '确认取消吗')
        while True:
            if inpt.is_pressed_short(inpt.b):
                oled.fill(0)
                gui.auto_show_text(1, "正  在  打  印")  # ---GUI---
                return
            time.sleep_ms(50)
